@@ -2,6 +2,7 @@ from typing import List, Dict, Optional
 import time, json
 
 from selenium_firefox.firefox import Firefox, By, Keys
+from kcu import rand
 
 from .url_creator import UrlCreator
 
@@ -41,20 +42,33 @@ class Pinterest:
     def follow(self, user_name: str) -> bool:
         try:
             self.browser.get(self.url_creator.user_url(user_name))
-            time.sleep(0.5)
+            rand.sleep(0.5, 1)
 
-            follow_container = self.browser.find(By.XPATH, "//div[contains(@class, 'CreatorFollowButton step0')]")
+            follow_container = self.browser.find(By.XPATH, "//div[contains(@class, 'Jea XiG zI7 iyn Hsu')]")
             follow_button = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ erh tg7 mWe')]", follow_container)
-            print(follow_button.text)
             if follow_button is not None and follow_button.text == "Follow":
                 follow_button.click()
-                time.sleep(0.5)
+                rand.sleep(0.2,0.5)
             else:
                 return False
-                
-            follow_button = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ erh tg7 mWe')]", timeout=3) or self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ pBj tg7 mWe')]", timeout=3)
+
+            follow_button = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ erh tg7 mWe')]", follow_container, timeout=3)
             
-            return follow_button.text == "Following"
+            if follow_button.text == "Following":
+                return True
+
+            if follow_button is None or follow_button.text != "Following":
+                print('im in this if')
+                follow_container_after_follow = self.browser.find(By.XPATH, "//div[contains(@class, 'Eqh Jea gRy jx- zI7 iyn Hsu')]")
+                container_elements = self.browser.find_all(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ pBj tg7 mWe')]", follow_container_after_follow)
+
+                for elem in container_elements:
+                    print(elem.text)
+                    if elem.text == "Following":
+
+                        return True
+
+            return False
 
         except Exception as e:
             print(e)
@@ -63,16 +77,20 @@ class Pinterest:
 
     def unfollow(self, user_name: str) -> bool:
         try:
-            self.browser.get(self.url_creator.user_url(user_name)) 
-            user = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ pBj tg7 mWe')]")
-            time.sleep(0.5)
-            if user is not None and user.text == "Following":
-                user.click()
-                time.sleep(0.5)
-            else:
-                return False
+            self.browser.get(self.url_creator.user_url(user_name))
+            rand.sleep(0.5, 1)
+            user_elements = self.browser.find_all(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ pBj tg7 mWe')]")
+
+            if user_elements is not None:
+                for user in user_elements:
+                    if user.text == "Following":
+                        user.click()
+                        rand.sleep(0.5, 1)
+                else:
+                    return False
                 
             user = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ erh tg7 mWe')]") or self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ pBj tg7 mWe')]")
+            print('user.text:', user.text)
             
             return user.text == "Follow"
         
@@ -84,12 +102,13 @@ class Pinterest:
     def repin(self, pin_id: str, board_name: str) -> bool:
         try:
             self.browser.get(self.url_creator.pin_url(pin_id))
-            time.sleep(1)
+            rand.sleep(0.7, 1.2)
 
             board_dropdown = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc _yT pBj DrD IZT swG z-6')]", timeout=3)
 
             if board_dropdown is not None:
                 board_dropdown.click()
+                rand.sleep(0.1, 0.5)
             else:
                 return self.__create_and_save_to_board(board_name)
 
@@ -98,17 +117,18 @@ class Pinterest:
             for board in boards:
                 if board.text == board_name:
                     board.click()
+                    rand.sleep(0.2,0.5)
                     break
             else:
                 self.browser.find(By.XPATH, "//div[contains(@class, 'rDA wzk zI7 iyn Hsu')]").click() # create board button
                 text_tag = self.browser.find(By.XPATH, "//input[contains(@id, 'boardEditName')]")
                 text_tag.send_keys(board_name)
-                time.sleep(0.5)
+                rand.sleep(0.5, 1)
                 self.browser.find(By.XPATH, "//button[contains(@class, 'RCK Hsu USg Vxj aZc Zr3 hA- GmH adn Il7 Jrn hNT iyn BG7 NTm KhY')]").click() # create_button
 
                 return True
 
-            time.sleep(1)
+            rand.sleep(0.5, 1)
 
             return self.browser.find(By.XPATH, "//div[contains(@class, 'Eqh Shl s7I zI7 iyn Hsu')]") is not None
         
@@ -123,10 +143,11 @@ class Pinterest:
             full_link = self.browser.find(By.CSS_SELECTOR, 'a', saved_to_button).get_attribute('href')
 
             self.browser.get(full_link)
-            time.sleep(1)
+            rand.sleep(1.5, 2)
 
             latest_image_box = self.browser.find(By.XPATH, "//div[contains(@class, 'Yl- MIw Hb7')]", timeout=5)
             pin_id = self.browser.find(By.XPATH, "//div[contains(@data-test-id, 'pin')]", latest_image_box).get_attribute('data-test-pin-id')
+            rand.sleep(0.1, 0.3)
 
             if pin_id is not None:
                 return pin_id
@@ -143,9 +164,9 @@ class Pinterest:
             self.browser.find(By.XPATH, "//div[contains(@class, 'Umk fte zI7 iyn Hsu')]").click() # create_board_button
             text_tag = self.browser.find(By.XPATH, "//input[contains(@id, 'boardEditName')]")
             text_tag.send_keys(board_name)
-            time.sleep(0.5)
+            rand.sleep(0.5, 0.8)
             self.browser.find(By.XPATH, "//button[contains(@class, 'RCK Hsu USg Vxj aZc Zr3 hA- GmH adn Il7 Jrn hNT iyn BG7 NTm KhY')]").click() # create_button
-            time.sleep(1)
+            rand.sleep(1, 1.5)
 
             return self.browser.find(By.XPATH, "//button[contains(@class, 'RCK Hsu USg Vxj aZc Zr3 hA- GmH adn Il7 Jrn hNT iyn BG7 NTm KhY')]") is None
         
@@ -167,13 +188,13 @@ class Pinterest:
             else:
                 self.browser.get(self.url_creator.board_url(user_name, board_name))
 
-            time.sleep(1)
+            rand.sleep(1, 1.5)
             followers_container = self.browser.find_all(By.XPATH, "//span[contains(@class, 'tBJ dyH iFc yTZ pBj DrD IZT mWe')]")
 
             for elem in followers_container:
                 if elem is not None and 'followers' in elem.text:
                     elem.click()
-                    time.sleep(1)
+                    rand.sleep(1, 1.5)
 
             saved_users = 0
             final_users = []
@@ -204,8 +225,8 @@ class Pinterest:
                     return (final_users, ignored_users)
                 
                 see_more_button.click()
-                time.sleep(0.5)
-            
+                rand.sleep(1, 1.5)
+
         except Exception as e:
             print(e)
 
@@ -214,7 +235,7 @@ class Pinterest:
     def search_pinterest_boards(self, search_term: str, number_of_boards_to_get: int=35) -> Optional[List[tuple]]:
         try:
             self.browser.get(self.url_creator.search_board_url(search_term))
-            time.sleep(1)
+            rand.sleep(1, 1.5)
 
             if self.browser.find(By.XPATH, "//div[contains(@class, 'noResults')]"):
                 return None
@@ -242,7 +263,7 @@ class Pinterest:
                         return board_urls
                     
                 self.browser.scroll(1000)
-                time.sleep(0.5)
+                rand.sleep(0.5, 1.5)
                 after_scroll = self.browser.current_page_offset_y()
 
                 if after_scroll == before_scroll:
@@ -256,10 +277,11 @@ class Pinterest:
     def get_pins_from_home_feed(self) -> Optional[List[str]]:
         try:
             self.browser.get(self.url_creator.home_feed_url())
-            time.sleep(1)
+            rand.sleep(1, 1.5)
 
             home_pins = []
             home_pin_containers = self.browser.find_all(By.XPATH, "//div[contains(@class, 'Yl- MIw Hb7')]")
+
             for pin in home_pin_containers:
                 full_url = self.browser.find(By.CSS_SELECTOR, 'a', pin).get_attribute('href')
 
@@ -286,11 +308,12 @@ class Pinterest:
         ) -> str:
         try:
             self.browser.get(self.url_creator.pin_builder_url())
-            time.sleep(1)
+            rand.sleep(1, 1.5)
 
             select_board_button = self.browser.find(By.XPATH, "/html/body/div[1]/div[1]/div[3]/div/div/div/div[2]/div/div/div/div/div/div/div/div[1]/div/div[2]/div/div/div")
             if select_board_button is not None:
                 select_board_button.click()
+                rand.sleep(0.1, 0.5)
             
             boards = self.browser.find_all(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ pBj DrD IZT mWe z-6')]") 
 
@@ -298,6 +321,8 @@ class Pinterest:
                 for board in boards:
                     if board.text == board_name:
                         board.click()
+                        rand.sleep(0.1, 0.5)
+
                         break
                 else:
                     dropdown_boards = self.browser.find(By.XPATH, "//div[contains(@class, 'DUt qJc sLG zI7 iyn Hsu')]")
@@ -305,12 +330,13 @@ class Pinterest:
 
                     if create_board is not None:
                         create_board.click()
+                        rand.sleep(0.1, 0.5)
                     
                     text_tag = self.browser.find(By.XPATH, "//input[contains(@id, 'boardEditName')]")
                     text_tag.send_keys(board_name)
-                    time.sleep(0.5)
+                    rand.sleep(0.5, 1)
                     self.browser.find(By.XPATH, "//button[contains(@class, 'RCK Hsu USg F10 xD4 fZz hUC GmH adn Il7 Jrn hNT iyn BG7 NTm KhY')]").click()
-                    time.sleep(0.5)
+                    rand.sleep(0.5, 1)
                     just_created_board_save_bttn = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ erh DrD IZT mWe')]")
                     just_created_board_save_bttn.click()
 
@@ -318,30 +344,34 @@ class Pinterest:
             if title_box is not None:
                 title = self.browser.find(By.CSS_SELECTOR, "textarea", title_box)
                 title.send_keys(title_text)
+                rand.sleep(0.2, 0.5)
             
             if about_text is not None:
                 about_box = self.browser.find(By.XPATH, "//div[contains(@class, 'Jea Tte ujU xcv L4E zI7 iyn Hsu')]")
                 if about_box is not None:
                     about = self.browser.find(By.CSS_SELECTOR, "textarea", about_box)
                     about.send_keys(about_text)
+                    rand.sleep(0.2, 0.5)
+
 
             if destination_link_text is not None:
                 destination_link_box = self.browser.find(By.XPATH, "//div[contains(@data-test-id, 'pin-draft-link')]")
                 if destination_link_box is not None:
                     destination_link = self.browser.find(By.CSS_SELECTOR, "textarea", destination_link_box)
                     destination_link.send_keys(destination_link_text)
+                    rand.sleep(0.2, 0.5)
             
             image_box = self.browser.find(By.XPATH, "//div[contains(@class, 'DUt XiG zI7 iyn Hsu')]")
             if image_box is not None:
                 image = self.browser.find(By.CSS_SELECTOR, 'input', image_box)
                 image.send_keys(file_path)
-                time.sleep(1.5)
+                rand.sleep(1, 1.5)
             
             save_button = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ erh DrD IZT mWe')]")
             if save_button is not None:
                 save_button.click()
             
-            time.sleep(0.1)
+            rand.sleep(0.1, 0.5)
             see_it_now = self.browser.find(By.XPATH, "//div[contains(@data-test-id, 'seeItNow')]")
             full_pin_url = self.browser.find(By.CSS_SELECTOR, 'a', see_it_now).get_attribute('href')
             pin_id = full_pin_url.split('pin/')[1]
@@ -358,11 +388,10 @@ class Pinterest:
             not_now = self.browser.find(By.XPATH, "//div[contains(@class, 'tBJ dyH iFc yTZ erh tg7 mWe')]")
             if not_now.text == "Not now":
                 not_now.click()
-            self.browser.find(By.XPATH, "//div[contains(@class, 'RCK Hsu USg Vxj aZc Zr3 hA- qQ7 YbY hNT iyn BG7 NTm KhY')]").click()
 
             return True
         
         except Exception as e:
             print(e)
 
-            return False
\ No newline at end of file
+            return False
